{ config, lib, pkgs, ... }:

{
	imports = [
		./hardware-configuration.nix
		../../modules
	];

	riley = {
		gui = true;
		kak = {

    		enable = true;

    		haskell = true;
    		rust = true;
    		nix = true;

		};
	};

	system.stateVersion = "21.11";

	networking = {
    	hostName = "thor";
		interfaces.enp9s0.useDHCP = true;
	};

    boot = {
        loader.systemd-boot = {
            enable = true;
            editor = false;
            configurationLimit = 10;
        };

        loader.efi = {
            canTouchEfiVariables = true;
        };
    };

	users.users."riley".packages = with pkgs; [
		minecraft
		cockatrice
	];

    devices = {

        # Audio devices
        audio = {

            # Outputs
            speakers = "alsa_output.pci-0000_0c_00.4.analog-stereo";
            external = "alsa_output.pci-0000_0a_00.1.hdmi-stereo";
            headset  = null; # Thor doesn't have bluetooth support (yet)

			# Inputs
            main-mic = "alsa_input.usb-046d_C922_Pro_Stream_Webcam_CC9E75BF-02.analog-stereo";
            
        };

        # Displays
        video.displays = {

            # Connected to the ultrawide output
            "HDMI-1" = {
                primary = true;
                position = [ 0 0 ];
            };

            # Vertical output to the right of HDMI-1
            "DP-2" = {
                rotate = "left";
                position = [ 2560 ((1080 - 1920) / 2) ];
            };
            
        };

    };

}
